        PAGE    60,132
NAME    IsWin
        TITLE   IsWin Version 1.3
COMMENT 

IsWin C:MyProg.Exe
Detect if program is a Windows program.
  IsWin C:MyProg.Exe
  If ErrorLevel 3 GoTo Problem
  If ErrorLevel 2 GoTO IsForWin32
  If ErrorLevel 1 GoTo IsForWin16
  Goto IsForDos

This code is based EXE header layouts documented in Volume 4 of
the Microsoft Windows Development kit.

A Windows GUI program has the following characteristics:
1. "MZ" at offset 0.  (also true of MS DOS EXE files.)
2. word at offset 18h is 40h or greater.
   This points to the first relocation item.
3. For pklite offset 1E is "PKLITE", a dos file.
4. The word at offset 3Ch is the offset to the windows header.
5. At offset 0 of the win16 header is the signature "NE" (New Executable)
   If it is Win32 program the signature is "PE" (Portable Executable)

Ray Duncan describes DOS header format

Win95 resource guide for NE header format?

describes Win32 PE header format:
http://mup.anticrack.de/Fatboy%20Joe%20-%20PE%20Format.htm

Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#101 - 2536 Wark Street
Victoria, BC Canada V8T 4G8
tel:(250) 361-9093
mailto:roedyg@mindprod.com
http://mindprod.com

Version 1.1 1996 October 25
- embed POB 707 Quathiaski Cove address
version 1.3 2003 August 14
- add detect for Win32 executables

Futures
- echo result to screen
- display syntax info if user enters a blank command line.

       ; end of comment

;======
stack   segment stack           ; keep MS link happy by providing null stack
stack   ends

;==============================================================

CODE    SEGMENT PARA            ; start off in code.

;==============================================================

data    segment word            ; provide a separate DATA segment
                                ; Even though it appears in the source
                                ; before the code, it the COM file it

;=============================================
;       E R R O R   M E S S A G E S
;=============================================

Buffer  DW      0

CopyrightMsg    label   byte
        db 13,10
        db      ' IsWin 1.3 ۲',13,10
        db      'Determines if the specified program is a Windows GUI program.',13,10
        DB      'Copyright: (c) 1993-2011 Roedy Green, Canadian Mind Products.',13,10
        DB      '#101 - 2536 Wark Street, Victoria, BC Canada V8T 4G8',13,10
        DB      'tel:(250) 361-9093   mailto:roedyg@mindprod.com   http://mindprod.com',13,10
        db      'May be freely distributed and used for any purpose except military.',13,10
        db      13,10
 db 'Error in command line.  Try:',13,10
 db '  IsWin C:MyProg.Exe',13,10
 db '  If ErrorLevel 3 GoTo Problem',13,10
 db '  If ErrorLevel 2 GoTo IsForWin32 (Win95/98/NT/2K/XP)',13,10
 db '  If ErrorLevel 1 GoTo IsForWin16 (Win3.1)',13,10
 db '  Goto IsForDos',13,10,'$'

data    endS

;==============================================================

com     group   code,data

        ASSUME  CS:COM,DS:COM,ES:NOTHING
        ORG     100H
Start:
        Call    Parse           ; prepare the command line
        jcxz    Failed

        Call    TestIsWin

        mov     ah,4ch          ; exit back to DOS
        int     21h

Failed:
        lea     dx,Copyrightmsg
        call    say
        mov     ax,4c03h        ; exit back to DOS with error code 3        int     21h

;==============================================================

TestIswin       proc    Near
;       ES:bx cx points to ASCIIZ filename.
;       an exit ax=0 -- DOS file, poss com, exe, PKLITE
;                  1 -- win16
;                  2 -- win32
;                  3 -- no such prog
        mov     dx,bx           ; DS:dx points to filename
        mov     ax,3d40h        ; open 0 100 0 000 mode deny none, readonly
        int     21h
        jc      NoSuchProg

; ------------------------------ READ THE MZ
        mov     bx,ax           ; handle
        mov     cx,2            ; read two bytes
        lea     dx,Buffer
        mov     ah,3fh          ; read operation
        int     21h
        cmp     ax,cx
        jne     DosProg

        cmp     Buffer,5A4Dh    ; is it MZ signature?
        jne     DosProg    ; com file no MZ signature

; ------------------------------ READ POINTER AT 18H

        mov     ax,4200h        ; lseek from start
        mov     dx,18h          ; offset CX:DX
        mov     cx,0
        int     21h
        jc      NoSuchProg

        mov     cx,2            ; read two bytes
        lea     dx,Buffer
        mov     ah,3fh          ; read operation
        int     21h
        cmp     ax,cx
        jne     NoSuchProg
        cmp     Buffer,40h      ; should be >= 40h
        jb      DosProg

; ------------------------------ READ signature at 1Eh
        mov     ax,4200h        ; lseek from start
        mov     dx,1Eh          ; offset CX:DX
        mov     cx,0
        int     21h
        jc      NoSuchProg
        mov     cx,2            ; read two bytes
        lea     dx,Buffer
        mov     ah,3fh          ; read operation
        int     21h
        cmp     ax,cx
        jne     NoSuchProg

        cmp     buffer,4B50h    ; is it  "PK' signature
        je      DosProg    ; PKLITE DOS

; ------------------------------ READ POINTER AT 3Ch

        mov     ax,4200h        ; lseek from start
        mov     dx,3ch          ; offset CX:DX
        mov     cx,0
        int     21h
        jc      NoSuchProg

        mov     cx,2            ; read two bytes
        lea     dx,Buffer
        mov     ah,3fh          ; read operation
        int     21h
        cmp     ax,cx
        jne     NoSuchProg    ; Will point to Windows header.

; ------------------------------ READ POINTER NE SIGNATURE usually at 400h

        mov     ax,4200h        ; lseek from start
        mov     dx,Buffer       ; offset CX:DX
        mov     cx,0
        int     21h
        jc      NoSuchProg

        mov     cx,2            ; read two bytes
        lea     dx,Buffer
        mov     ah,3fh          ; read operation
        int     21h
        cmp     ax,cx
        jne     NoSuchProg

        cmp     buffer,454Eh    ; is it 'NE' signature
        je      Win16
        cmp     buffer,4550h    ; is it 'PE' signature
        je      win32
        jmp     NoSuchProg

DosProg:
        mov     ax,0
        ret

Win16:
        mov     ax,1
        ret

Win32:
        mov     ax,2
        ret

NoSuchProg:
        mov     ax,3
        ret

TestIswin       EndP

;=============================================
;       C O M M A N D   L I N E   P A R S I N G   R O U T I N E S
;=============================================

MLeading        PROC    Near

;       Remove leading blanks
;       on entry ES:BX is addr of string, CX its length
;       trims off any leading blanks, leaving result in ES:BX CX
;       length may also be 0 or 1, but not -ve
;       If the entire string is blank the result is the null string

        mov     di,bx
        mov     al,20H          ; AL = blank  -- the search char
        jcxz    mleading2       ; jump if null string
        repe    scasb           ; scan ES:DI forwards till hit non blank
                                ; DI points just after it (wrap ok)
                                ; cx IS ONE TOO SMALL, OR 0 IF NONE FOUND
        je      mleading1       ; jump if entire string was blank
        inc     cx              ; CX is length of remainder of string
mleading1:
        dec     di              ; DI points to non-blank
mleading2:
        mov     bx,di           ; put address back
        ret

MLeading        ENDP

;========================================

MTrailing       PROC    Near

;       Remove trailing blanks.
;       on entry ES:BX is addr of string, CX its length
;       trims off any trailing blanks, leaving result in BX CX
;       length may also be 0 or 1, but not -ve
;       If the entire string is blank the result is the null string

        mov     di,bx
        add     di,cx           ; calc addr last char in string
        dec     di
        mov     al,20H          ; AL = blank  -- the search char
        jcxz    mtrailing1      ; jump if null string
        std
        repe    scasb           ; scan ES:DI backwards till hit non blank
                                ; DI points just ahead of it (wrap ok)
                                ; CX is one too small, or 0 if none found
        cld
        je      mtrailing1      ; jump if whole string was blank
        inc     cx
mtrailing1:
        ret

MTrailing       ENDP

;========================================

Parse           PROC    NEAR
;       Parse the command line to remove lead/trail blanks from
;       the single drive parameter and terminate it by 2 nulls.
;       sample inputs
;       IsWin A:MyFile.exe
;       IsWin   C:MyFile.exe
;       IsWin
;       ES: points to PSP
;       When Done ES:BX points to start of string.
;       String will be terminated by 2 nulls
;       CX counts bytes in string exclusive of nulls
                                ; counted string at HEX 80 PSP
                                ; contains command line.
                                ; Preceeded by unwanted spaces.
                                ; possibly followed by unwanted spaces.
                                ; currently missing a trailing null.
        xor     ch,ch
        mov     cl,es:[80H]
        mov     bx,81H
        call    Mleading        ; get rid of leading blanks
        call    MTrailing       ; get rid of trailing blanks
        mov     di,bx           ; calc addr of byte just past end
        add     di,cx
        mov     word ptr ES:[di],0 ; plop in pair of nulls after string
        ret

Parse           ENDP

;======================================

Say             Proc    Near
;       on entry DX points to a string to display on screen
        MOV     AH,9
        Int     21h
        ret
Say             EndP

;=======================================

CODE ends
end start
